#!/bin/bash

# Version 15/11/2012
# $Revision: 1.5 $
# By UDA'Software 2012

# Modulo contenente le Funzioni Standard di elaborazione degli XBSPSE.

#####################################################################
#____________________________________________________________________

# Funzione per ottenere la versione del plugin.

function get_buduscript_version() {
    [ -f "$(dirname "${0}")/version.c" ] && \
	grep -iF 'Version' "$(dirname "${0}")/version.c" | grep -o '[0-9]*\.[0-9]*\.[0-9]*' | tail -n1 | sed 's@\.@@g' || \
	echo "/ECHO ERRORE: Impossibile determinare la versione dello script" >&2
}

#####################################################################
#____________________________________________________________________

# Funzione per la creazione del DFE (Data File Exchange)

# Gli XBSPSE di tipo LIST e PASSWORD generano un database permanente
# che contiene tutti i records disponibili; questo database deve
# essere filtrato per lasciare esclusivamente i record che soddisfano
# le chiavi di ricerca dell'utente, in modo da ottenere solo i
# risultati desiderati.

function make_data_file_exchange() {
    search_result=$(cat /tmp/budus_XDCC_db_${plugin_name}.${date_code})
    for pattern in ${SEARCH_STRING}; do
	search_result=$(echo "${search_result}" | grep -iF "${pattern}")
    done
    
    echo "${search_result}" > ${plugin_tempfile}.database
}

#####################################################################
#____________________________________________________________________

# Funzione per calcolare le variabili base operative degli XBSPSE.

# Il parametro passato alla funzione - non facoltativo - deve essere
# il path completo dell'XBSPSE (di norma e' sufficiente passare
# "${0}").

function make_plugin_variables() {
    plugin_start_time=$(date +%s)
    date_code=$(date +%Y%m%d)
    plugin_path=$(dirname "${1}")
    plugin_name=$(echo "${1}" | sed 's@.*budus_xdcc_plugin_@@' | cut -f1 -d'.')
    user_home=$(echo "${HOME}" | tr [:punct:] '_')
    plugin_tempfile="/tmp/budus_XDCC_${plugin_name}_${user_home}_${RANDOM_NUMBER}"
}

#####################################################################
#____________________________________________________________________

# Funzione per ottenere la Password necessaria per accedere alla
# LISTA.

# Viene utilizzato un "crawler" che effettua le seguenti operazioni:
# - Si collega al network/canale
# - Verifica la presenza del BOTSERV che deve fornire la password
# - Interroga il BOTSERV fornendogli una parola chiave (in genere un
#   link)
# - Attende la risposta (la password o comunque la stringa che la
#   contiene)

# Per funzionare la funzione prevede che siano preventivamente
# configurate tutte queste variabili:

# crawler_server_host="irc.NETWORK.dominio"
# crawler_server_port="6667"
# crawler_server_channel="#CANALE"
# crawler_bot_nickname="BOTSERV_PASSWORD_NICKNAME"
# crawler_greeting_string="Ciao a tutti quanti"
# crawler_query_nickname="${crawler_bot_nickname}"
# crawler_query_string="KEYWORD_O_URL"
# crawler_string_pattern="PATTERN_INDIVIDUAZIONE_PASSWORD"

function get_list_password() {
    printf '/ECHO [XBSPSE PLUGIN]\t %s: Per accedere al database delle risorse XDCC devo interrogare il BOT [%s:%s:%s] per ottenere la password.\n' "${plugin_name}" "${crawler_server_host}" "${crawler_server_channel}" "${crawler_bot_nickname}"
    
    {
	. "${plugin_path}/budus_XBSPSE_Crawler.sh"
	XBSPSE_Crawler_ID=$(get_XBSPSE_ID)
	
	connect_to_irc_server "${plugin_name}" "${crawler_server_host}" "${crawler_server_port}" "${crawler_server_channel}"
	irc_connection_error
	[ $? == 0 ] && \
	    irc_connection_manager "${crawler_bot_nickname}" "${crawler_greeting_string}" "${crawler_query_nickname}" "${crawler_query_string}" "${crawler_string_pattern}"
    } > "${plugin_tempfile}"
    
    [ -f "${plugin_tempfile}" ] && XBSPSE_Crawler=$(cat "${plugin_tempfile}")
    rm -f "${plugin_tempfile}"
}

#####################################################################
#____________________________________________________________________

# FILTRI

# Di seguito sono presenti le funzioni da utilizzare per filtrare la
# lista scaricata, in modo da creare il database permanente, sul
# quale, effettuare le ricerche.

# La scelta del FILTRO va effettuata in base al formato della
# lista. Purtroppo non e' semplice - e forse possibile - stabilire a
# priori il formato e quindi bisogna procedere empiricamente (cioe':
# si guarda il sorgente della lista e, in base all'esperienza, si
# capisce con quale filtro puo' essere elaborata) o per tentativi (si
# provano tutti i filtri fino a quando non si ottiene un database
# "pulito").

#####################################################################
#____________________________________________________________________

# Filtro: ONCLICK_generic <NETWORK> <SERVER> <CHANNEL>

function filter_ONCLICK_generic() {
    cat ${plugin_tempfile} \
	| grep -F ' OnClick=' \
	| sed 's@.*<tr.* OnClick="t(.@'"${1}"'\t'"${2}"'\t'"${3}"'\t@g' \
	| sed "s@','@\t@g" \
	| sed 's@.);".\?href=.*</a></th><td width=[0-9]\{1,3\}%>@\t@g' \
	| sed 's@</td><td width=[0-9]\{1,3\}% align=left>@\t@g' \
	| sed 's@</td></tr>.*@@g' \
	| sed 's@</\?font\( color=[^>]*\)\?>@@g' \
	| sed 's@^<.*>@@' \
	| sed 's@&lt;@<@g' \
	| awk -F'\t' '{print $1"\t"$2"\t"$3"\t"$4"\t"$5"\t"$7"\t"$6"\t/msg "$4" xdcc send "$5}' \
	> /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: ONCLICK_IA <NETWORK> <SERVER> <CHANNEL>

function filter_ONCLICK_IA() {
    cat ${plugin_tempfile} \
	| grep -F ' OnClick=' \
	| sed 's@.*<tr.* OnClick="t(.@'"${1}"'\t'"${2}"'\t'"${3}"'\t@g' \
	| sed "s@','#@\t#@g" \
	| sed "s@');.href=#.* width=[0-9]*%>@\t@g" \
	| sed 's@<td width=[0-9]*% align=left>@\t@g' \
	| sed 's@</td></tr>$@@g' \
	| sed 's@<font color=\(#[0-9a-fA-F]\{6\}\|[a-zA-Z]*\)>[ ]\?@@g' \
	| sed 's@ \?</font> \?@@g' \
	| sed 's@</td>.*$@@g' \
	| grep -v ^$ \
	| awk -F'\t' '{print $1"\t"$2"\t"$3"\t"$4"\t"$5"\t"$7"\t"$6"\t/msg "$4" xdcc send "$5}' \
	> /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: ONCLICK_Fantasy <NETWORK> <SERVER> <CHANNEL>

function filter_ONCLICK_Fantasy() {
    cat ${plugin_tempfile} \
	| grep -F ' OnClick=' \
	| sed 's@.*<tr.* OnClick="t(.@'"${1}"'\t'"${2}"'\t'"${3}"'\t@g' \
	| sed "s@','@\t@g" \
	| sed "s@');.href=#.* width=[0-9]*%>\[\?@\t@g" \
	| sed 's@\]\?<td width=[0-9]*% align=left>@\t@g' \
	| sed 's@</td></tr>$@@g' \
	| sed 's@<font color=\(#[0-9a-fA-F]\{6\}\|[a-zA-Z]*\)>[ ]\?@@g' \
	| sed 's@ \?</font> \?@@g' \
	| sed 's@</td>.*$@@g' \
	| grep -v ^$ \
	| grep '	#[0-9]*	' \
	| awk -F'\t' '{print $1"\t"$2"\t"$3"\t"$4"\t"$5"\t"$7"\t"$6"\t/msg "$4" xdcc send "$5}' \
	> /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: ONCLICK_Eden <NETWORK> <SERVER> <CHANNEL>

function filter_ONCLICK_Eden() {
    cat ${plugin_tempfile} \
	| grep -F ' OnClick=' \
	| sed 's@.*<tr.* OnClick="t(.@'"${1}"'\t'"${2}"'\t'"${3}"'\t@g' \
	| sed "s@','@\t@g" \
	| sed 's@.);" href=.*</a></td><td id=b>@\t@g' \
	| sed 's@<td id=c>@\t@g' \
	| sed 's@</td></tr>.*@@g' \
	| sed 's@</\?font\( color=[^>]*\)\?>@@g' \
	| sed 's@^<.*>@@' \
	| sed 's@&lt;@<@g' \
	| awk -F'\t' '{print $1"\t"$2"\t"$3"\t"$4"\t"$5"\t"$7"\t"$6"\t/msg "$4" xdcc send "$5}' \
	> /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: ONCLICK_OneLine <NETWORK> <SERVER> <CHANNEL>

function filter_ONCLICK_OneLine() {
    cat ${plugin_tempfile} \
	| sed 's@<th width=[0-9]*%><a OnClick=@\nBUDUS_XBSPSE_RECORD:@g' \
	| grep -F 'BUDUS_XBSPSE_RECORD:' \
	| sed 's@BUDUS_XBSPSE_RECORD:"t(.@'"${1}"'\t'"${2}"'\t'"${3}"'\t@g' \
	| sed "s@','@\t@g" \
	| sed "s@');.href=#.* width=[0-9]*%>@\t@g" \
	| sed 's@<font color=#[0-9a-fA-F]*>[^0-9]\?@@' \
	| sed 's@[^0-9]\?</font><td width=[0-9]*% align=left><a OnClick=.*href=#>@\t@' \
	| sed 's@<font color=\(#[0-9a-fA-F]\{6\}\|[a-zA-Z]*\)> @@g' \
	| sed 's@ \?</font> \?@@g' \
	| sed 's@</td>.*$@@g' \
	| grep -v -e '^$' -e '</table>' \
	| awk -F'\t' '{print $1"\t"$2"\t"$3"\t"$4"\t"$5"\t"$7"\t"$6"\t/msg "$4" xdcc send "$5}' \
	> /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: ONCLICK_Horror <NETWORK> <SERVER> <CHANNEL>

function filter_ONCLICK_Horror() {
    cat ${plugin_tempfile} \
	| sed 's@<th width=[0-9]*%><a OnClick=@\nBUDUS_XBSPSE_RECORD:@g' \
	| grep -F 'BUDUS_XBSPSE_RECORD:' \
	| sed 's@BUDUS_XBSPSE_RECORD:"t(.@'"${1}"'\t'"${2}"'\t'"${3}"'\t@g' \
	| sed "s@','@\t@g" \
	| sed "s@');.href=#.* width=[0-9]*%>@\t@g" \
	| sed 's@<td width=[0-9]*% align=left>@\t@' \
	| sed 's@<font color=#[0-9a-fA-F]*>[^0-9]\?@@' \
	| sed 's@ \?</font> \?@@g' \
	| sed 's@</td>.*$@@g' \
	| grep -v -e '^$' -e '</table>' \
	| awk -F'\t' '{print $1"\t"$2"\t"$3"\t"$4"\t"$5"\t"$7"\t"$6"\t/msg "$4" xdcc send "$5}' \
	> /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: filter_COPYTOCLIPBOARD_generic <NETWORK> <SERVER> <CHANNEL>

function filter_COPYTOCLIPBOARD_generic() {
    cat ${plugin_tempfile} \
	| sed 's@<div id="line">@\n@g' \
	| grep -F '<div id="pack_id">' \
	| sed 's@</div><div id="download">.*@@' \
	| sed 's@.*copy_to_clipboard(.@@' \
	| sed 's@\(.);"><b>\|</b></a></div><div id="dimension">.\|.</div><div id="packs">\)@\t@g' \
	| sed 's@\(/msg \| xdcc send #[0-9]*\)@@g' \
	| sed 's@" href=.*</div>@@' \
	| awk -F'\t' '{print "'"${1}"'\t'"${2}"'\t'"${3}"'\t"$1"\t"$2"\t"$4"\t"$3"\t/msg "$1" xdcc send "$2}' \
	> /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: MULTISTAGE_generic <NETWORK> <SERVER> <CHANNEL>

function filter_MULTISTAGE_generic() {
    {
	NETWORK_NAME="${1}"
	NETWORK_HOST="${2}"
	CHANNEL="${3}"
	
	IFS=$'\n'
	for record in $(cat ${plugin_tempfile} \
	    | sed 's@<table class="lista"><tr><th colspan="4">@\nBUDUS_BOT_NAME@g' \
	    | grep -F 'BUDUS_BOT_NAME' \
	    | sed 's@^BUDUS_BOT_NAME@@g' \
	    | sed 's@</th>@\t@g'); do
	    BOT_NAME=$(echo "${record}" | awk -F'\t' '{print $1}')
	    
	    echo "${record}" \
		| awk -F'\t' '{print $2}' \
		| sed 's@<a href=".">@\n@g' \
		| sed 's@</a></td></tr><tr><td>@\t@g' \
		| sed 's@ *</td><td>@\t@g' \
		| sed 's@</td><td class="data">.*@@g' \
		| sed 's@&lt;@<@g' \
		| grep -v '<.*>' \
		| awk -v NETWORK_NAME="${NETWORK_NAME}" -v NETWORK_HOST="${NETWORK_HOST}" -v CHANNEL="${CHANNEL}" -v BOT_NAME="${BOT_NAME}" -F'\t' '{print NETWORK_NAME"\t"NETWORK_HOST"\t"CHANNEL"\t"BOT_NAME"\t"$2"\t"$1"\t"$3"\t/MSG "BOT_NAME" XDCC SEND "$2}'
	done
    } > /tmp/budus_XDCC_db_${plugin_name}.${date_code}
}

#####################################################################
#____________________________________________________________________

# Filtro: MULTISTAGE_generic <NETWORK> <SERVER> <CHANNEL>

function filter_MULTISTAGE_friend4ever() {
    {
	NETWORK_NAME="${1}"
	NETWORK_HOST="${2}"
	CHANNEL="${3}"
	
	cat ${plugin_tempfile} \
	    | sed 's@^[^<]*@@' \
	    | grep -F '<table class="lista"><tr><th colspan="4">' \
	    | sed 's@<table class="lista"><tr><th colspan="4">@\n#BOTNAME\t@g' \
	    | sed 's@</th></tr><tr class="header"><td>Pack</td><td>Dimen.</td><td>Data</td><td style="width: 100%;">Nome File</td></tr><tr><td>@\n@' \
	    | sed 's@ \?</td><td>@\t@g' \
	    | sed 's@</td><td class="data">@\t@g' \
	    | sed 's@<a href="#">@\t@g' \
	    | sed 's@</a></td></tr><tr><td>@\n@g' \
	    | sed 's@</a></td></tr></table></div>\(</div>\)\?@@g' \
	    | sed 's@<div class="xdcc" style="display: block;">@@g' \
	    | sed 's@&lt;1k@1k@g' \
	    | grep '^#' > /tmp/budus_XDCC_db_${plugin_name}.${date_code}.stage1
	
	filter_MULTISTAGE_friend4ever_stage1_link=$(grep -n '^#BOTNAME	' /tmp/budus_XDCC_db_${plugin_name}.${date_code}.stage1 \
	    | awk -F':' '{print $1}')
	filter_MULTISTAGE_friend4ever_stage1_link=$(printf '%s\n%d\n' "${filter_MULTISTAGE_friend4ever_stage1_link}" $(($(cat /tmp/budus_XDCC_db_${plugin_name}.${date_code}.stage1 | wc -l) + 1)))
	
	IFS=$'\n'
	for link_end in ${filter_MULTISTAGE_friend4ever_stage1_link}; do
	    if [ "${link_begin}" != "" ]; then
		BOT_NAME=$(sed -n ${link_begin},${link_begin}p /tmp/budus_XDCC_db_${plugin_name}.${date_code}.stage1 \
		    | awk -F'\t' '{print $2}')
		
		link_begin=$((${link_begin} + 1))
		link_end=$((${link_end} - 1))
		
		sed -n ${link_begin},${link_end}p /tmp/budus_XDCC_db_${plugin_name}.${date_code}.stage1 \
		    | awk -v NETWORK_NAME="${NETWORK_NAME}" -v NETWORK_HOST="${NETWORK_HOST}" -v CHANNEL="${CHANNEL}" -v BOT_NAME="${BOT_NAME}" -F'\t' '{print NETWORK_NAME"\t"NETWORK_HOST"\t"CHANNEL"\t"BOT_NAME"\t"$1"\t"$5"\t"$2"\t/MSG "BOT_NAME" XDCC SEND "$1}'
		
		link_begin=$((${link_end} + 1))
	    else
		link_begin=${link_end}
	    fi
	done
    } > /tmp/budus_XDCC_db_${plugin_name}.${date_code}
    
    rm -f /tmp/budus_XDCC_db_${plugin_name}.${date_code}.stage1
}

#####################################################################
#____________________________________________________________________

# Funzione per il test dei Filtri (NON USARE)

function check_xbspse_filter() {
    filter_name=${1}

    printf 'Vuoi eseguire il test del filtro %s ? (S/n/q) ' "${filter_name}"
    read answer
    
    if [ "${answer}" == "q" ] || [ "${answer}" == "Q" ]; then
	rm -f /tmp/budus_XDCC_db_${plugin_name}.${date_code}
	
	exit 0;
    fi
    
    if [ "${answer}" != "n" ] && [ "${answer}" != "N" ]; then
	printf '\n* ANALISI FILTRO: %s\n' "${filter_name}"
	
	${filter_name} 'NETWORK' 'SERVER' '#CHANNEL'
	
	db_records_total=$(cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" | wc -l)
	db_records_find=$(cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" \
	    | grep '^[^ ]*	[^ ]*	#.*	[^ ]*	#\?[0-9]*[ ]\{0,\}	[^"]*	[ <0-9\.]*[a-zA-Z]\{0,2\}	/.*[0-9] \{0,\}$' \
	    | grep -v '<.*>' \
	    | wc -l)
	
	if [ ${db_records_total} -gt 0 ]; then
	    successfull_average=$(($((${db_records_find} * 100)) / ${db_records_total}))
	    successfull_average_1000=$(($((${db_records_find} * 1000)) / ${db_records_total:-1}))
	else
	    successfull_average=0
	    successfull_average_1000=0
	fi
	
	if [ ${db_records_total:-0} -gt ${db_records_find:-0} ] && [ ${successfull_average_1000:-0} -lt 1000 ]; then
	    printf '\nESTRAPOLATI %d RECORDS CERCANDO SU UN DATABASE PERMANENTE DI %d RECORDS (%d%%).\n' ${db_records_find:-0} ${db_records_total:-0} ${successfull_average:-0}
	    
	    {
		cat<<EOF
********************************************************************************
RECORDS MALFORMATI NEL DATABASE PERMANENTE DI: ${plugin_name}:${filter_name}
FILE DATABASE PERMANENTE: "/tmp/budus_XDCC_db_${plugin_name}.${date_code}"

RECORDS TOTALI     : $(printf '%9d\n' ${db_records_total:-0})
RECORDS CORRETTI   : $(printf '%9d\n' ${db_records_find:-0})
RECORDS MALFORMATI : $(printf '%9d\n' $((${db_records_total:-0} - ${db_records_find:-0})))
RAPPORTO SUCCESSO  : $(printf '%10s\n' ${successfull_average}%)
********************************************************************************
Premi il tasto 'q' per uscire e usa i tasti freccia per spostarti su e giu.
********************************************************************************
EOF
		
		[ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ] && \
		    cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" \
		    | grep -v '^[^ ]*	[^ ]*	#.*	[^ ]*	#\?[0-9]*[ ]\{0,\}	[^"]*	[ <0-9\.]*[a-zA-Z]\{0,2\}	/.*[0-9] \{0,\}$'
		
		cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" \
		    | grep '<.*>'
		
		cat<<EOF
********************************************************************************
Premi il tasto 'q' per uscire e usa i tasti freccia per spostarti su e giu.
********************************************************************************
EOF
	    } | less
	else
	    if [ ${db_records_total:-0} -gt 0 ]; then
		printf '\nESTRAPOLATI %d RECORDS CERCANDO SU UN DATABASE PERMANENTE DI %d RECORDS (%d%%).\n' ${db_records_find:-0} ${db_records_total:-0} ${successfull_average:-0}
		
		{
		    cat<<EOF
********************************************************************************
RECORDS CORRETTI NEL DATABASE PERMANENTE DI: ${plugin_name}:${filter_name}
FILE DATABASE PERMANENTE: "/tmp/budus_XDCC_db_${plugin_name}.${date_code}"

RECORDS TOTALI     : $(printf '%9d\n' ${db_records_total:-0})
RAPPORTO SUCCESSO  : $(printf '%10s\n' ${successfull_average}%)
********************************************************************************
Premi il tasto 'q' per uscire e usa i tasti freccia per spostarti su e giu.
********************************************************************************
EOF
		    
		    [ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ] && \
			cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}"
		    
		    cat<<EOF
********************************************************************************
Premi il tasto 'q' per uscire e usa i tasti freccia per spostarti su e giu.
********************************************************************************
EOF
		} | less
	    else
		printf '\\_ FALLIMENTO TOTALE !!!\n'
	    fi
	fi
    else
	printf '\n* FILTRO SCARTATO: %s\n' "${filter_name}"
    fi
}

#####################################################################
#____________________________________________________________________

# ROUTINE DI TEST (USARE SOLO A RIGA DI COMANDO PER CREAZIONE XBSPSE)

# E' possibile invocare questo script al fine di testare una LISTA
# scaricata con tutti i filtri disponibili ed individuare quello che
# produce i risultati migliori.
if [ "${1}" == "--filter-test" ]; then
    plugin_tempfile="${2}"
    [ ! -f "${plugin_tempfile}" ] && \
	echo 'ERRORE: File lista da analizzare non specificato.' && \
	exit 1
    
    clear
    printf 'ANALISI DEL FILE: %s\n' "${plugin_tempfile}"
    
    plugin_name=budus_XBSPSE_LIB
    date_code=TEST

    check_xbspse_filter filter_ONCLICK_generic
    
    check_xbspse_filter filter_ONCLICK_IA
    
    check_xbspse_filter filter_ONCLICK_Fantasy
    
    check_xbspse_filter filter_ONCLICK_Eden
    
    check_xbspse_filter filter_ONCLICK_OneLine
    
    check_xbspse_filter filter_ONCLICK_Horror
    
    check_xbspse_filter filter_COPYTOCLIPBOARD_generic
    
    check_xbspse_filter filter_MULTISTAGE_generic
    
    check_xbspse_filter filter_MULTISTAGE_friend4ever
    
    rm -f /tmp/budus_XDCC_db_${plugin_name}.${date_code}
fi

#####################################################################
#____________________________________________________________________