#!/bin/bash

# Version 10/06/2012
# $Revision: 1.33 $
# UDA'Software 2010

# Questo script viene eseguito al primo avvio di BuDuScRiPt o
# successivamente ad un aggiornamento.
#
# Il suo scopo e' quello di eseguire le operazioni necessarie per
# verificare che non ci siano anomalie note in grado di compromettere
# il funzionamento del plugin e di informare l'utente circa le ultime
# novita' introdotte.

#####################################################################
#____________________________________________________________________
# Disabilita il file globbing
set -f

#####################################################################
#____________________________________________________________________
config_file="${HOME}/.xchat2/buduscript/budus_external_menu.config"

#####################################################################
#____________________________________________________________________
script_path="$(dirname $0)"

# Carica la libreria
source "${script_path}/budus_xdcc.sh" LIBRARY

cat "${script_path}/ChangeLog" \
    | LC_ALL=C grep -nF '********************************************************************' \
    > /tmp/buduscript_script__first.dat
last_line_changelog=$(head -n2 /tmp/buduscript_script__first.dat \
    | tail -n1 \
    | cut -f1 -d:)
rm -f /tmp/buduscript_script__first.dat

IFS_OLD=${IFS}
IFS=$'\n'

#####################################################################
#____________________________________________________________________
# Stampa il Changelog
printf '/ECHO %%C7,1%%B/---------------{ %%C3,1ELENCO MODIFICHE NUOVA VERSIONE%%C7,1 }----------------\\\n'
changelog_data="$(head -n${last_line_changelog:-1} "${script_path}/ChangeLog" | sed 's@^@/ECHO %C8,1%B@g')"
for line in ${changelog_data}; do
    printf '%s\n' "${line}"
done

#####################################################################
#____________________________________________________________________
# Messaggio ulteriori info
printf '/ECHO %%C3,1%%B Per ulteriori informazioni (ad esempio quelle relative alle\n'
printf '/ECHO %%C3,1%%Bprocedure esterne) o richieste di aiuto consultare il Forum:\n'
printf '/ECHO %%C9,1%%B                  http://budus.ilbello.com/Forum\n'
printf '/ECHO %%C0,1%%B                  (NON E'\'' RICHIESTA ISCRIZIONE)\n'
printf '/ECHO %%C3,1%%Boppure utilizzare il comando di menu:\n'
printf '/ECHO %%C9,1%%B        BuDuScRiPt > Aiuto > Ottenere Supporto ON-LINE\n'
printf '/ECHO %%C3,1%%B********************************************************************\n'

#####################################################################
#____________________________________________________________________
# Verifica il BUG della macro autounban.mcr
if [ -f "${script_path}/autounban.mcr" ]; then
    find_bug_autounban_macro=$(grep -F '/budus else /BAN $4' "${script_path}/autounban.mcr")
else
    printf '/ECHO %%C0,1%%BMissing MACRO: autounban.mcr\n'
fi
if [ "${find_bug_autounban_macro}" != "" ]; then
    echo '/budus PlaySound() e'
    
    cat <<EOF | sed 's@^@/ECHO %C4,1%B@g'
ATTENZIONE: Le versioni di BuDuScRiPt precedenti alla 3.2.03
            contenevano un bug relativo alla protezione contro i
            BAN\KICK sull'utente locale nelle procedure esterne al
            plugin.
            Dato che nel sistema e' stato rilevato il file
            "autounban.mcr" contenente il bug e' necessario lanciare
            il seguente comando da XChat:
            %C9,1/EXEC rm -vf ${script_path}/autounban.mcr
            e quindi procedere a crearne uno nuovo tramite la voce
            di menu: %C6,1BuDuScRiPt > Preferenze 
            %C6,1> Protezione BAN\KICK > Crea la MACRO AutoUnBan di base
********************************************************************
EOF
fi

#####################################################################
#____________________________________________________________________
# Verifica configurazione XDCC.
config_file="${script_path}/budus_xdcc.config"
if [ -f "${config_file}" ]; then
    if [ "$(grep -F 'ABILITA_RECUPERO_MANUALE' "${config_file}")" != "" ]; then
	cat <<EOF | sed 's@^@/ECHO %C7,1%B@g'
ATTENZIONE: Le versioni di BuDuScRiPt fino alla 3.4.46 facevano uso
            di una configurazione dell'XDCC MANAGER che supportava
            la variabile "ABILITA_RECUPERO_MANUALE"; ora questa
            variabile e' stata sostituita da altre due.
            Si rende necessario verificare manualmente, tramite il 
            menu XDCC, che la configurazione attuale risponda alle
            proprie esigenze.
********************************************************************
EOF
    fi
fi

#####################################################################
#____________________________________________________________________
# Elimina i XBSPSE obsoleti.
XBSPSE_list="$(cat "${script_path}/buduscript_xbspse_obsolete.dat")"

flag_found_old_XBSPSE=0
for XBSPSE in ${XBSPSE_list}; do
    if [ -f "${script_path}/${XBSPSE}" ]; then
	printf '/ECHO %%C8,1%%B[XDCC MANAGER]\t%%O%%C8,1 Intercettato XBSPSE obsoleto, provvedo ad eliminarlo:%%B %s\n' "${XBSPSE/\.sh/}"
	mv -f "${script_path}/${XBSPSE}" "${script_path}/${XBSPSE/\.sh/}.old"
	rm -f "${script_path}/${XBSPSE/\.sh/}"
	flag_found_old_XBSPSE=1
    fi
done

if [ ${flag_found_old_XBSPSE} == 1 ]; then
    cat<<EOF | sed 's@^@/ECHO %C8,1%B@g'
********************************************************************
Sono stati intercettati degli XBSPSE (XDCC Bash Script Plugin Search
Engine) obsoleti che sono stati rimossi e disabilitati in automatico
********************************************************************
EOF
    get_var XDCC_MENU 1
    XDCC_MENU=${get_var}
    if [ ${XDCC_MENU:-0} == 1 ]; then
	flag_skip_next_reload_xdcc_menu=1
	load_menu_XBSPSE
	cat<<EOF | sed 's@^@/ECHO %C8,1%B@g'
Il menu dei XBSPSE disponibili e' stato ricaricato al fine di
aggiornare la situazione.
********************************************************************
EOF
    fi
fi

#####################################################################
#____________________________________________________________________
# Ripristina i XBSPSE obsoleti.
XBSPSE_list="$(cat "${script_path}/buduscript_xbspse_restore.dat")"

flag_found_old_XBSPSE=0
for XBSPSE in ${XBSPSE_list}; do
    if [ -f "${script_path}/${XBSPSE}" ]; then
	printf '/ECHO %%C8,1%%B[XDCC MANAGER]\t%%O%%C8,1 Intercettato XBSPSE non obsoleto, lo riesumo:%%B %s\n' "${XBSPSE/\.sh/}"
	rm -f "${script_path}/${XBSPSE}"
	flag_found_old_XBSPSE=1
    fi
done

if [ ${flag_found_old_XBSPSE} == 1 ]; then
    cat<<EOF | sed 's@^@/ECHO %C8,1%B@g'
********************************************************************
Sono stati intercettati degli XBSPSE (XDCC Bash Script Plugin Search
Engine) non piu' obsoleti che sono stati ripristinati.
********************************************************************
EOF
    get_var XDCC_MENU 1
    XDCC_MENU=${get_var}
    if [ ${XDCC_MENU:-0} == 1 ]; then
	[ ${flag_skip_next_reload_xdcc_menu:-0} == 0 ] && load_menu_XBSPSE
	cat<<EOF | sed 's@^@/ECHO %C8,1%B@g'
Il menu dei XBSPSE disponibili e' stato ricaricato al fine di
aggiornare la situazione.
********************************************************************
EOF
    fi
fi

#####################################################################
#____________________________________________________________________
# Verifica se ci sono degli XBSPSE disabilitati.
get_var XDCC_MENU 1
XDCC_MENU=${get_var}
if [ ${XDCC_MENU:-0} == 1 ]; then
    deactive_plugin_list=$(find "${script_path}" -iname 'budus_xdcc_plugin_*' \
	| grep -v '\.old$' \
	| sed 's@\.sh$@@' \
	| sort \
	| uniq -u)
    
    for XBSPSE in ${deactive_plugin_list}; do
	if [ ! -f "${script_path}/${XBSPSE}" ]; then
	    printf '/ECHO %%C8,1%%B[XDCC MANAGER]\t%%O%%C8,1 Intercettato XBSPSE non attivo:%%B %s\n' "$(basename ${XBSPSE} | sed 's@.*budus_xdcc_plugin_@@' | tr '_' ' ')"
	fi
    done
fi

#####################################################################
#____________________________________________________________________
# Verifica la presenza del file di configurazione per twitter; se non
# c'e', lo crea in automatico.
if [ ! -f "${HOME}/.xchat2/buduscript/twitter.cfg" ] && \
    [ ! -f "${HOME}/.xchat2/buduscript/twitter.cfg.lock" ] && \
    [ ! -f "${HOME}/.xchat2/buduscript/twitter.cfg.deactivated" ]; then
    cat<<EOF | sed 's@^@/ECHO %C7,1%B@g'
********************************************************************
Non risulta presente il file di configurazione del servizio che
gestisce le news provenienti da Twitter.
Verra' creata in automatico una configurazione che segue "budus_it".
Per informazioni, consultare la documentazione del plugin alla voce
Twitter.
********************************************************************
EOF
    cat<<EOF > "${HOME}/.xchat2/buduscript/twitter.cfg"
# File di Configurazione di Twitter ($(date))
#
# Per ogni riga puoi configurare un account twitter dal quale
# ricevere i tweets. La sintassi da utilizzare e' la seguente:
#
# <account_name> <echo | send [network/]channel> [filter]
#
# dove "account_name" e' il nome dell'account twitter dal quale
# ricevere i messaggi; "echo" e' il comando per visualizzare i
# messaggi ricevuti privatamente mentre "send" li invia
# pubblicamente al "network" - facoltativo - e canale ("channel")
# specificati.
# Esiste, inoltre, la possibilita' di specificare il percorso
# completo ad un file eseguibile (o comunque uno script) che,
# sfruttando il meccanismo della PIPE del Sistema Operativo, vada a
# filtrare l'output di ogni tweet.
#
# Esempi:
#
# budus_it echo
# budus_it send #buduscript
# budus_it send irc.mynetwork.net/#buduscript
# budus_it send irc.mynetwork.net/#buduscript /home/x/my_filter.sh
#
# Le righe che iniziano con "#" sono di commento e vengono ignorate.
#
####################################################################
budus_it echo
EOF
fi

#####################################################################
#____________________________________________________________________
# Verifica che il file di personalizzazione degli annunci MP3 sia
# aggiornato alla versione 3.5.56
budus_mp3_command_file="${script_path}/budus_mp3_command.txt"
if [ -f "${budus_mp3_command_file}" ]; then
    if [ "$(grep '^FILE:' "${budus_mp3_command_file}")" == "" ]; then
	cat<<EOF | sed 's@^@/ECHO %C7,1%B@g'
********************************************************************
E' stato individuato il file di configurazione che personalizza gli
annunci delle canzoni in ascolto con i player multimediali
supportati (${budus_mp3_command_file}).

Tuttavia sembra che non sia conforme a quanto richiesto a partire
dalla versione 3.5.56 di BuDuScRiPt.

Sarebbe opportuno rinominarlo (ex: /EXEC mv -vf ${budus_mp3_command_file} ${budus_mp3_command_file}.$(date +%Y%m%d%H%M%S) ),
utilizzare il comando di menu:

BuDuScRiPt -> Preferenze -> MP3 - Parametri dei Player Audio -> Personalizza Annunci ...

e modificare quello nuovo prendendo spunto dal vecchio.

Si consiglia di leggere la FAQ dello script per ulteriori
informazioni.
********************************************************************
EOF
    fi
fi

#####################################################################
#____________________________________________________________________
# Apre la pagina di ringraziamento con il browser, se lo trova.
current_date=$(date +%Y%m%d)
if [ ! -f ${script_path}/budus_first_inibition ]; then
    if [ -f ${script_path}/budus_first_inibition.browser ]; then
	thanks_you_url="http://digilander.libero.it/udasoft/Software/Linux/buduscript/thanks_you.html"
	
	last_update=$(tail -n1 ${script_path}/budus_first_inibition.browser)
	if [ $((${current_date:-1} - ${last_update:-0})) -gt 90 ]; then
	    if [ $(which x-www-browser > /dev/null 2> /dev/null) ]; then
		echo "/TIMER 55 EXEC x-www-browser '${thanks_you_url}' > /dev/null 2> /dev/null &"
	    elif [ $(which xdg-open > /dev/null 2> /dev/null) ]; then
		echo "/TIMER 55 EXEC xdg-open '${thanks_you_url}' > /dev/null 2> /dev/null &"
	    elif [ $(which firefox 2> /dev/null) ]; then
		echo "/TIMER 55 EXEC firefox '${thanks_you_url}' > /dev/null 2> /dev/null &"
	    elif [ $(which google-chrome 2> /dev/null) ]; then
		echo "/TIMER 55 EXEC google-chrome '${thanks_you_url}' > /dev/null 2> /dev/null &"
	    elif [ $(which opera 2> /dev/null) ]; then
		echo "/TIMER 55 EXEC opera '${thanks_you_url}' > /dev/null 2> /dev/null &"
	    fi
	else
	    printf '/TIMER 55 ECHO [BUDUSCRIPT]\t Se lo ritieni opportuno, lascia un feedback su questo plugin; vedi menu:%%B BuDuScRiPt -> Aiuto -> Rilascia un Feedback\n'
	fi
    else
	printf '/TIMER 55 ECHO [BUDUSCRIPT]\t Se lo ritieni opportuno, lascia un feedback su questo plugin; vedi menu:%%B BuDuScRiPt -> Aiuto -> Rilascia un Feedback\n'
    fi
else
    printf '/TIMER 55 ECHO [BUDUSCRIPT]\t Se lo ritieni opportuno, lascia un feedback su questo plugin; vedi menu:%%B BuDuScRiPt -> Aiuto -> Rilascia un Feedback\n'
fi

echo "${current_date}" >> ${script_path}/budus_first_inibition.browser

#####################################################################
#____________________________________________________________________
# Auto-Elimina il budus_first
echo "/TIMER 60 EXEC rm -f ${script_path}/budus_first"

#####################################################################
#____________________________________________________________________