#!/bin/bash

# Version 09/11/2012
# $Revision: 1.10 $
# By UDA'Software 2011

# Questo script ha lo scopo di verificare il funzionamento degli
# XBSPSE dell'XDCC MANAGER.

#####################################################################
#____________________________________________________________________
script_path=$(dirname "${0}")

clear
until [ -z "$1" ]
do
    case "$1" in
	"--help")
	    printf 'PARAMETRI DISPONIBILI:\n'
	    cat<<EOF

  --check-only-active-xbspse		Controlla solo gli XBSPSE attivi.

  --check-only-db			Controlla solo i database permanenti presenti.

  --check-only-web-xbspse		Controlla solo gli XBSPSE di tipo WEB.

  --do-not-ask-user			Inibisce l'interazione con l'utente.

  --do-not-remove-permanent-db		Non rimuove i database permanenti.

  --do-not-remove-db			Non rimuove i database di scambio dati.

  --help				Visualizza questa guida.

  --keyword <string>			Utilizza la stringa specificata per eseguire la ricerca.

  --min-malformed-record-average <num>	Percentuale (in millesimi) di successo minima consentita.

  --skip-db-check			I database permanenti non vengono controllati.

  --skip-password-xbspse		Non controlla gli XBSPSE di tipo PASSWORD.

  --skip-web-xbspse			Non controlla gli XBSPSE di tipo WEB.

  --test-only-xbspse <xbspse>		Verifica esclusivamente l'XBSPSE specificato.
                                        Se non specificato, elenca quelli disponibili.

  --view-malformed-record		Visualizza eventuali record malformati.

  --xbspse-timeout <secondi>		Imposta il TIMEOUT ai secondi specificati.

EOF
	    
	    exit 0;
	    ;;
	"--check-only-active-xbspse")
	    check_only_active_xbspse=1
	    printf 'VERRANNO CONTROLLATI SOLO GLI XBSPSE ATTIVI.\n'
	    ;;
	"--check-only-db")
	    check_only_db=1
	    printf 'VERRANNO CONTROLLATI SOLO I DATABASE PERMANENTI PRESENTI, SALTANDO I CRAWLER.\n'
	    ;;
	"--check-only-web-xbspse")
	    if [ ${check_only_db:-0} == 1 ]; then
		printf 'ATTENZIONE: IMPOSSIBILE CONTROLLARE XBSPSE DI TIPO WEB QUANDO E'\'' ATTIVATA L'\''OPZIONE "--check-only-db".\n'
	    else
		check_only_web_xbspse=1
		printf 'VERRANNO CONTROLLATI SOLO GLI XBSPSE DI TIPO WEB.\n'
	    fi
	    ;;
	"--do-not-ask-user")
	    do_not_ask_user=1
	    printf 'ATTIVATA MODALITA'\'' NON INTERATTIVA.\n'
	    ;;
	"--do-not-remove-permanent-db")
	    do_not_remove_permanent_db=1
	    printf 'IL DATABASE PERMANENTE DEGLI XBSPSE DI TIPO LIST/PASSWORD NON SARA'\'' ELIMINATO.\n'
	    ;;
	"--do-not-remove-db")
	    do_not_remove_db=1
	    printf 'IL DATABASE DI SCAMBIO DATI NON SARA'\'' ELIMINATO.\n'
	    ;;
	"--keyword")
	    shift
	    if [ "${1}" == "" ]; then
		printf 'ERRORE: MANCA IL PARAMETRO STRINGA PER L'\''OPZIONE "--keyword".\n'
		exit 1;
	    else
		KEYWORD="${1}"
		
		printf 'KEYWORD IMPOSTATA A %s.\n' ${KEYWORD:-NULL}
	    fi
	    ;;
	"--min-malformed-record-average")
	    shift
	    if [ "$(echo "${1}" | grep '^[0-9]*$')" == "" ]; then
		printf 'ERRORE: MANCA IL PARAMETRO XBSPSE PER L'\''OPZIONE "--min-malformed-record-average".\n'
		
		exit 1;
	    else
		min_successfull_average_1000="${1}"
		
		printf 'PERCENTUALE SOTTO LA QUALE UN DATABASE PERMANENTE RISULTA MALFORMATO: %d%% .\n' "$((${min_successfull_average_1000} / 10))"
		
		view_malformed_record=1
		printf 'EVENTUALI RECORDS MALFORMATI SARANNO OPPORTUNAMENTE VISUALIZZATI.\n'
	    fi
	    ;;
	"--skip-db-check")
	    if [ ${check_only_db:-0} == 1 ]; then
		printf 'ATTENZIONE: IMPOSSIBILE SALTARE IL CONTROLLO DEI DATABASE PERMANENTI QUANDO E'\'' ATTIVATA L'\''OPZIONE "--check-only-db".\n'
	    else
		skip_db_check=1
		printf 'IL CONTROLLO DEI DATABASE PERMANENTI SARA'\'' SALTATO.\n'
	    fi
	    ;;
	"--skip-password-xbspse")
	    skip_password_xbspse=1
	    printf 'GLI XBSPSE DI TIPO PASSWORD NON SARANNO CONTROLLATI.\n'
	    ;;
	"--skip-web-xbspse")
	    skip_web_xbspse=1
	    printf 'GLI XBSPSE DI TIPO WEB NON SARANNO CONTROLLATI.\n'
	    ;;
	"--test-only-xbspse")
	    shift
	    if [ "${1}" == "" ]; then
		printf 'ERRORE: MANCA IL PARAMETRO XBSPSE PER L'\''OPZIONE "--test-only-xbspse".\n'
		printf '\nElenco gli XBSPSE disponibili:\n'
		ls "${script_path}/budus_xdcc_plugin_"*".sh" | sed 's@.*budus_xdcc_plugin_@@g' | sed 's@\.sh$@@g'
		
		exit 1;
	    else
		XBSPSE_to_test="${1}"
		
		printf 'CONTROLLO DEL SOLO XBSPSE %s .\n' "${XBSPSE_to_test}"
	    fi
	    ;;
	"--view-malformed-record")
	    if [ ${do_not_ask_user:-0} == 1 ]; then
		printf 'ATTENZIONE: IMPOSSIBILE VERIFICARE I RECORDS MALFORMATI QUANDO E'\'' ATTIVATA L'\''OPZIONE "--do-not-ask-user".\n'
	    elif [ ${view_malformed_record:-0} == 0 ]; then
		view_malformed_record=1
		printf 'EVENTUALI RECORDS MALFORMATI SARANNO OPPORTUNAMENTE VISUALIZZATI.\n'
	    fi
	    ;;
	"--xbspse-timeout")
	    shift
	    if [ "${1}" == "" ]; then
		printf 'ERRORE: MANCA IL PARAMETRO SECONDI PER L'\''OPZIONE "--xbspse-timeout".\n'
		exit 1;
	    else
		TIMEOUT="${1}"
		
		TIMEOUT=$(echo "${TIMEOUT}" | grep -o '^[0-9]*$')
		[ ${TIMEOUT:-0} -le 0 ] && TIMEOUT=600
		printf 'TIMEOUT IMPOSTATO A %d.\n' ${TIMEOUT:-600}
	    fi
	    ;;
	*)
	    printf 'PARAMETRO %s NON RICONOSCIUTO.\n' "$1"
	    ;;
    esac
    
    shift
done

#####################################################################
#____________________________________________________________________
xbspse_list=$(ls "${script_path}/budus_xdcc_plugin_"*".sh")
IFS=$'\n'

date +"%F %R"
start_time=$(date +%s)
xbspse_num=0
db_records_total_sum=0

for xbspse in ${xbspse_list}; do
    if [ "${XBSPSE_to_test}" != "" ]; then
	is_XBSPSE_to_check=$(echo "${xbspse}" | grep -iF "budus_xdcc_plugin_${XBSPSE_to_test}.sh")
    else
	is_XBSPSE_to_check="*"
    fi

    if [ "${is_XBSPSE_to_check}" != "" ]; then
	date_code=$(date +%Y%m%d)
	plugin_path=$(dirname "${0}")
	plugin_name=$(echo "${xbspse}" | sed 's@.*budus_xdcc_plugin_@@' | cut -f1 -d'.')
	user_home=$(echo "${HOME}" | tr [:punct:] '_')
	plugin_tempfile=/tmp/budus_XDCC_${plugin_name}_${user_home}_TEST
	
	printf '\n\n\nVERIFICA XBSPSE: %32s\n' "${plugin_name}"
	
	if [ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ]; then
	    db_records_total=$(cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" | wc -l)
	    if [ ${db_records_total:-0} -le 1 ]; then
		printf 'INTERCETTATO DATABASE PERMANENTE NULLO; LO ELIMINO IN AUTOMATICO.\n'
		rm -vf "/tmp/budus_XDCC_db_${plugin_name}.${date_code}"
	    fi
	fi
	
	if [ ${check_only_db:-0} == 0 ]; then
	    [ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ] && \
		[ ${do_not_remove_permanent_db:-0} == 0 ] && \
		rm -vf "/tmp/budus_XDCC_db_${plugin_name}.${date_code}"
	fi
	
	plugin_type="$(grep -F "# PLUGIN TYPE:" ${xbspse} | cut -f2- -d':')"
	
	flag_disable_check=0
	if [ ${check_only_active_xbspse:-0} == 1 ]; then
	    if [ ! -f "${xbspse/.sh/}" ]; then
		flag_disable_check=1
	    fi
	fi
	
	if [ ${check_only_db:-0} == 0 ]; then
	    if [ ${flag_disable_check} == 0 ]; then
		if [ "${plugin_type}" != "WEB" ]; then
		    if [ ${check_only_web_xbspse:-0} == 0 ]; then
			if [ "${plugin_type}" == "PASSWORD" ] && [ ${skip_password_xbspse:-0} == 1 ]; then
			    printf 'XBSPSE DI TIPO PASSWORD; SALTATO IL CONTROLLO.\n'
			    flag_no_report=1
			    flag_no_error_on_null_permanent_db=1
			    flag_no_error_on_null_result=1
			else
			    xbspse_num=$((${xbspse_num} + 1))
			    
			    if [ ${skip_db_check:-0} == 0 ]; then
				time "${xbspse}" TEST ${TIMEOUT:-600} "${KEYWORD:-.}"
			    else
				time "${xbspse}" BUDUSCRIPT_XBSPSE_SEARCH_ENGINE_QUERY ${TIMEOUT:-600}
				flag_no_error_on_null_result=1
				rm -vf "/tmp/budus_XDCC_${plugin_name}_${user_home}_BUDUSCRIPT_XBSPSE_SEARCH_ENGINE_QUERY.database"
			    fi
			fi
		    else
			printf 'XBSPSE DI TIPO %s; SALTATO IL CONTROLLO.\n' "${plugin_type}"
			flag_no_report=1
			flag_no_error_on_null_permanent_db=1
			flag_no_error_on_null_result=1
		    fi
		else
		    if [ ${skip_web_xbspse:-0} == 0 ]; then
			xbspse_num=$((${xbspse_num} + 1))
			
			if [ ${skip_db_check:-0} == 0 ]; then
			    time "${xbspse}" TEST ${TIMEOUT:-600} "${KEYWORD:-linux}"
			else
			    time "${xbspse}" BUDUSCRIPT_XBSPSE_SEARCH_ENGINE_QUERY ${TIMEOUT:-600}
			    flag_no_error_on_null_result=1
			    rm -vf "/tmp/budus_XDCC_${plugin_name}_${user_home}_BUDUSCRIPT_XBSPSE_SEARCH_ENGINE_QUERY.database"
			fi
			flag_no_error_on_null_permanent_db=1
		    else
			printf 'XBSPSE DI TIPO WEB; SALTATO IL CONTROLLO.\n'
			flag_no_report=1
			flag_no_error_on_null_permanent_db=1
			flag_no_error_on_null_result=1
		    fi
		fi
	    else
		printf 'XBSPSE DI TIPO %s; SALTATO IL CONTROLLO, RISULTA DISATTIVATO.\n' "${plugin_type}"
		flag_no_report=1
		flag_no_error_on_null_permanent_db=1
		flag_no_error_on_null_result=1
	    fi
	else
	    printf 'XBSPSE DI TIPO %s; SALTATO IL CRAWLER, RISULTA DISATTIVATO.\n' "${plugin_type}"
	    flag_no_report=1
	fi
	
	if [ ${skip_db_check:-0} == 0 ] && \
	    [ ${flag_disable_check} == 0 ]; then
	    if [ ${check_only_db:-0} == 0 ] || \
		([ ${check_only_db:-0} == 1 ] && [ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ]); then
		db_records_total=0
		if [ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ]; then
		    db_records_total=$(cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" | wc -l)
		elif [ -f "${plugin_tempfile}.database" ]; then
		    db_records_total=$(cat "${plugin_tempfile}.database" | wc -l)
		fi
		
		db_records_find=0
		if [ ${check_only_db:-0} == 0 ]; then
		    [ -f "${plugin_tempfile}.database" ] && \
			db_records_find=$(cat "${plugin_tempfile}.database" \
			| grep '^[^ ]*	[^ ]*	#.*	[^ ]*	#\?[0-9]*[ ]\{0,\}	[^"]*	[ <0-9\.]*[a-zA-Z]\{0,2\}	/.*[0-9] \{0,\}$' \
			| grep -v '<.*>' \
			| wc -l)
		else
		    [ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ] && \
			db_records_find=$(cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" \
			| grep '^[^ ]*	[^ ]*	#.*	[^ ]*	#\?[0-9]*[ ]\{0,\}	[^"]*	[ <0-9\.]*[a-zA-Z]\{0,2\}	/.*[0-9] \{0,\}$' \
			| grep -v '<.*>' \
			| wc -l)
		fi
		db_records_total_sum=$((${db_records_total_sum} + ${db_records_total}))
		
		if [ ${db_records_total} -gt 0 ]; then
		    successfull_average=$(($((${db_records_find} * 100)) / ${db_records_total}))
		    successfull_average_1000=$(($((${db_records_find} * 1000)) / ${db_records_total:-1}))
		else
		    successfull_average=0
		    successfull_average_1000=0
		fi
		
		[ ${flag_no_report:-0} == 0 ] && \
		    xbspse_report="$(printf '%s\nESECUZIONE %7s : %s (%d%%)' "${xbspse_report:-RAPPORTO $(date):}" "$([ ${successfull_average:-0} == 0 ] && echo 'FAILED' || echo 'OK')" "${plugin_name}" ${successfull_average:-0})" || \
		    xbspse_report="$(printf '%s\nESECUZIONE SALTATA : %s (%d%%)' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}" ${successfull_average:-0})"
		
		[ ${flag_no_report:-0} == 0 ] && \
		    printf '\nESTRAPOLATI %d RECORDS CERCANDO SU UN DATABASE PERMANENTE DI %d RECORDS (%d%%).\n' ${db_records_find:-0} ${db_records_total:-0} ${successfull_average:-0}
		
		flag_no_report=0
		
		if [ ${db_records_total:-0} -gt ${db_records_find:-0} ] && \
		    [ ${successfull_average_1000:-0} -lt ${min_successfull_average_1000:-999} ] && \
		    [ ${flag_no_error_on_null_permanent_db:-0} == 0 ]; then
		    if [ ${view_malformed_record:-0} == 1 ]; then
			{
			    cat<<EOF
********************************************************************************
RECORDS MALFORMATI NEL DATABASE PERMANENTE DI: ${plugin_name}
FILE DATABASE PERMANENTE: "/tmp/budus_XDCC_db_${plugin_name}.${date_code}"

RECORDS TOTALI     : $(printf '%9d\n' ${db_records_total:-0})
RECORDS CORRETTI   : $(printf '%9d\n' ${db_records_find:-0})
RECORDS MALFORMATI : $(printf '%9d\n' $((${db_records_total:-0} - ${db_records_find:-0})))
RAPPORTO SUCCESSO  : $(printf '%10s\n' ${successfull_average}%)
********************************************************************************
Premi il tasto 'q' per uscire e usa i tasti freccia per spostarti su e giu.
********************************************************************************
EOF
			    if [ ${check_only_db:-0} == 0 ]; then
				[ -f "${plugin_tempfile}.database" ] && \
				    cat "${plugin_tempfile}.database" \
				    | grep -v '^[^ ]*	[^ ]*	#.*	[^ ]*	#\?[0-9]*[ ]\{0,\}	[^"]*	[ <0-9\.]*[a-zA-Z]\{0,2\}	/.*[0-9] \{0,\}$'
				cat "${plugin_tempfile}.database" \
				    | grep '<.*>'
			    else
				[ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ] && \
				    cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" \
				    | grep -v '^[^ ]*	[^ ]*	#.*	[^ ]*	#\?[0-9]*[ ]\{0,\}	[^"]*	[ <0-9\.]*[a-zA-Z]\{0,2\}	/.*[0-9] \{0,\}$'
				cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" \
				    | grep '<.*>'
			    fi
			    
			    cat<<EOF
********************************************************************************
Premi il tasto 'q' per uscire e usa i tasti freccia per spostarti su e giu.
********************************************************************************
EOF
			} | less
			
			[ "${XBSPSE_to_test}" == "" ] && printf '\nVuoi continuare con l'\''analisi ? (S/n)\n'
			[ "${XBSPSE_to_test}" == "" ] && [ ${do_not_ask_user:-0} == 0 ] && read answer
			
			if [ ${answer:-s} == n ] || [ ${answer:-S} == N ]; then
			    rm -fv "${plugin_tempfile}"
			    [ ${do_not_remove_db:-0} == 0 ] && rm -fv "${plugin_tempfile}.database"
			    date +"%F %R"
			    end_time=$(date +%s)
			    
			    printf 'VERIFICATI %d XBSPSE in %d SECONDI PER UN TOTALE DI %d RECORDS PERMANENTI.\n' ${xbspse_num:-0} $((${end_time:-0} - ${start_time:-0})) ${db_records_total_sum:-0}
			    [ ${do_not_ask_user:-0} == 1 ] && sleep 4
			    
			    exit 0;
			fi
		    else
			xbspse_report="$(printf '%s\nRISULTATO DUBBIO  : %s (%d%%)' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}" ${successfull_average:-0})"
			
			printf 'ESECUZIONE DI XBSPSE %s DUBBIA; PREMI INVIO PER CONTINUARE.\n' "${plugin_name}"
			[ ${do_not_ask_user:-0} == 0 ] && read
		    fi
		elif [ ${db_records_total:-0} -le 1 ] && [ ${flag_no_error_on_null_permanent_db:-0} == 0 ]; then
		    xbspse_report="$(printf '%s\nRISULTATO FALLITO : %s' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}")"
		    
		    printf 'ESECUZIONE DI XBSPSE %s FALLITA; PREMI INVIO PER CONTINUARE.\n' "${plugin_name}"
		    [ ${do_not_ask_user:-0} == 0 ] && read
		elif [ ${db_records_find:-0} -le 1 ] && [ ${flag_no_error_on_null_result:-0} == 0 ]; then
		    xbspse_report="$(printf '%s\nRISULTATO FALLITO : %s' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}")"
		    
		    printf 'ESECUZIONE DI XBSPSE %s NULLA; PREMI INVIO PER CONTINUARE.\n' "${plugin_name}"
		    [ ${do_not_ask_user:-0} == 0 ] && read
		fi
	    else
		if [ "${plugin_type}" != "WEB" ]; then
		    if [ ${check_only_db:-0} == 0 ]; then
			xbspse_report="$(printf '%s\nRISULTATO FALLITO : %s' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}")"
			
			printf 'ESECUZIONE DI XBSPSE %s FALLITA, DATABASE PERMANENTE NON TROVATO; PREMI INVIO PER CONTINUARE.\n' "${plugin_name}"
			[ ${do_not_ask_user:-0} == 0 ] && read
		    else
			xbspse_report="$(printf '%s\nRISULTATO FALLITO : %s' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}")"
			
			printf 'CONTROLLO DI XBSPSE %s FALLITO, DATABASE PERMANENTE NON TROVATO; PREMI INVIO PER CONTINUARE.\n' "${plugin_name}"
			[ ${do_not_ask_user:-0} == 0 ] && read
		    fi
		fi
	    fi
	else
	    if [ -f "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" ]; then
		db_size=$(cat "/tmp/budus_XDCC_db_${plugin_name}.${date_code}" | wc -l)
		xbspse_report="$(printf '%s\n* %48s : %9s records' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}" ${db_size:-0})"
	    else
		db_size=$(cat "/tmp/budus_XDCC_db_${plugin_name}.dat" | wc -l)
		if [ ${db_size:-0} == 0 ]; then
		    db_size=$(stat --printf='%s' "/tmp/budus_XDCC_db_${plugin_name}.dat")
		    xbspse_report="$(printf '%s\n* %48s : %9s bytes' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}" ${db_size:-0})"
		else
		    xbspse_report="$(printf '%s\n* %48s : %9s linee' "${xbspse_report:-RAPPORTO $(date):}" "${plugin_name}" ${db_size:-0})"
		fi
	    fi
	fi
	
	rm -fv "${plugin_tempfile}"
	[ ${do_not_remove_db:-0} == 0 ] && rm -fv "${plugin_tempfile}.database"
    fi
done
date +"%F %R"
end_time=$(date +%s)

if [ -d "${script_path}/tmp" ]; then
    {
	printf 'VERIFICATI %d XBSPSE in %d SECONDI PER UN TOTALE DI %d RECORDS PERMANENTI.\n' ${xbspse_num:-0} $((${end_time:-0} - ${start_time:-0})) ${db_records_total_sum:-0}
	printf '\n%s\n' "${xbspse_report}"
    } > "${script_path}/tmp/buduscript_script__test_xbspse.log"
    [ -f "${script_path}/tmp/buduscript_script__test_xbspse.log" ] && \
	cat "${script_path}/tmp/buduscript_script__test_xbspse.log" || \
	echo "RAPPORTO NON PRESENTE."
else
    printf 'VERIFICATI %d XBSPSE in %d SECONDI PER UN TOTALE DI %d RECORDS PERMANENTI.\n' ${xbspse_num:-0} $((${end_time:-0} - ${start_time:-0})) ${db_records_total_sum:-0}
    printf '\n%s\n' "${xbspse_report}"
fi

[ ${do_not_ask_user:-0} == 1 ] && sleep 4

#####################################################################
#____________________________________________________________________
